import { world, BlockPermutation, ItemStack } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe((initEvent) => {
    initEvent.blockComponentRegistry.registerCustomComponent("rustic:wildberry_bush", {
        onRandomTick: e => {
            const { block, dimension } = e;
            const growState = block.permutation.getState("r:berries");

            if ( growState ) return;

            if ( (Math.random() * 100) > 15 ) return;

            block.setPermutation(block.permutation.withState("r:berries", !growState));
        },
        onPlayerInteract: e => {
            const { player, block, dimension, face, faceLocation } = e;
            const item = player.getComponent('inventory').container.getItem(player.selectedSlotIndex);
            const growState = block.permutation.getState("r:berries");

            if ( !growState ){
                if (item == undefined) return;
                if (item.typeId != 'minecraft:bone_meal') return;

                block.dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
                block.dimension.playSound("item.bone_meal.use", block.center());
                const equipment = player.getComponent('equippable');
                if (player.getGameMode() !== "creative") {
                    if (item.amount > 1) {
                        item.amount -= 1;
                        equipment.setEquipment('Mainhand', item);
                    } else if (item.amount === 1) {
                        equipment.setEquipment('Mainhand', undefined);
                    }
                }
                block.setPermutation(block.permutation.withState("r:berries", !growState));
            }
            else if ( growState ) {
                block.setPermutation(block.permutation.withState("r:berries", !growState));
                player.getComponent('inventory').container.addItem(new ItemStack("rustic:wildberries", 1));
            }
        },
        beforeOnPlayerPlace: e => {
            const { player, block, face, permutationToPlace, dimension } = e

            const random = Math.random() * 100

            let state = 0

            if ( random < 25 ) {
                state = 1
            }
            if ( random < 50 ) {
                state = 2
            }
            else if ( random < 75 ) {
                state = 3
            }
            else if ( random < 100 ) {
                state = 4
            }
            e.permutationToPlace = BlockPermutation.resolve(permutationToPlace.type.id,{"r:position": state, "r:firstGrow": true})
        },
        onTick: e => {
            const { block, dimension } = e;
            const isPlacedByPlayer = block.permutation.getState("r:firstGrow");
            if (isPlacedByPlayer) return;
            if (block.typeId == "minecraft:air") return;
            block.setPermutation(block.permutation.withState("r:berries", true));
            block.setPermutation(block.permutation.withState("r:firstGrow", true));
        }


    });
});

world.afterEvents.playerBreakBlock.subscribe(eventData => {

    if(eventData.player.getGameMode() == "creative" ) return;

    if (eventData.brokenBlockPermutation.type.id === "rustic:wildberry_bush") {

        const growStage = eventData.brokenBlockPermutation.getState("r:berries")

        if ( !growStage ) return;

        eventData.dimension.spawnItem(new ItemStack("rustic:wildberries"), eventData.block.center());
    }
});


export const berriesLoot = {
    pools: [{
        rolls: 1,
        entries: [{item:'rustic:wildberries'}]
    }]
}
