import { world, system, ItemStack } from "@minecraft/server";
import '../../spawn_loot/spawn_loot';
import { oliveSeeds, appleSeeds, sugar } from "./seedsLootTable";

world.beforeEvents.worldInitialize.subscribe((initEvent) => {
    initEvent.blockComponentRegistry.registerCustomComponent("rustic:crushing_tub", {
        onEntityFallOn: e => {
            const { entity, block, fallDistance, dimension } = e;
            const itemState = block.permutation.getState("r:place_item");
            const liquidState = block.permutation.getState("r:liquid");
            const typeState = block.permutation.getState("r:type");
            const particle = "rustic:" + typeState + "_particle"

            if (itemState == 0) return;
            if (fallDistance < 1) return;

            block.setPermutation(block.permutation.withState("r:place_item", itemState - 1));
            if (liquidState < 6) block.setPermutation(block.permutation.withState("r:liquid", liquidState + 1));
            block.dimension.runCommandAsync(`playsound fall.slime @a ${block.x} ${block.y} ${block.z} 10 0.8`)

            if (typeState == "apple") block.dimension.spawnLoot(appleSeeds, block.center())
            if (typeState == "olives") block.dimension.spawnLoot(oliveSeeds, block.center())
                if (typeState == "sugar_cane") block.dimension.spawnLoot(sugar, block.center())

            block.dimension.runCommandAsync(`particle ${particle} ${block.center().x} ${block.center().y} ${block.center().z}`)
            
        },
    });
});

//Crops Interaction
world.beforeEvents.playerInteractWithBlock.subscribe(eventData => {
    const block = eventData.block;
    const item = eventData.itemStack;
    const player = eventData.player;

    if (!block.hasTag("CrushingTub"))return;

    if (item == undefined) return;

    const blockState = block.permutation.getState("r:type");
    const state = item.typeId.split(":")[1];
    const itemState = block.permutation.getState("r:place_item");

    const isAllowed = {"wildberries":{},"ironberries":{},"olives":{},"apple":{},"grapes":{},"honeycomb":{},"sugar_cane":{}}
    
    if(!isAllowed[state])return;

    let stateNumber = 1
    if(blockState == 'none'){
        if(player.isSneaking)stateNumber = item.amount > 6 ? 6 : item.amount;
        system.run( () => {
            block.setPermutation(block.permutation.withState("r:type", state));
            block.setPermutation(block.permutation.withState("r:place_item", stateNumber));
            block.dimension.runCommandAsync(`playsound use.slime @a ${block.x} ${block.y} ${block.z} 10 0.8`)
            removeItem(player,item,stateNumber)
        })
    }
    else{
        if (blockState != state) return;
        if (itemState == 6) return;
        stateNumber = itemState + 1
        if(player.isSneaking && itemState == 0)stateNumber = item.amount > 6 ? 6 : item.amount;
        system.run( () => {
            block.setPermutation(block.permutation.withState("r:place_item", stateNumber));
            block.dimension.runCommandAsync(`playsound use.slime @a ${block.x} ${block.y} ${block.z} 10 0.8`)
            if(player.isSneaking && itemState != 0)stateNumber = 1;
            removeItem(player,item,stateNumber)
        })
    }

    eventData.cancel = true
});

//Hand Interaction
world.beforeEvents.playerInteractWithBlock.subscribe(eventData => {
    const block = eventData.block;
    const item = eventData.itemStack;
    const player = eventData.player;

    if (!block.hasTag("CrushingTub"))return;

    if (item != undefined) return;

    if (!player.isSneaking) return;

    const blockState = block.permutation.getState("r:type");
    const itemState = block.permutation.getState("r:place_item");
    const liquidState = block.permutation.getState("r:liquid");

    if (itemState < 1) return;

    if(blockState == 'none') return;

    let itemId = "rustic:"

    if (blockState == 'apple' || blockState == 'honeycomb' || blockState == 'sugar_cane'){
        itemId = "minecraft:"
    }

    itemId += blockState

    system.run( () => {
        if (itemState == 1 && liquidState == 0){
            block.setPermutation(block.permutation.withState("r:type", 'none'));
        }
        block.setPermutation(block.permutation.withState("r:place_item", itemState - 1));
        block.dimension.runCommandAsync(`playsound use.slime @a ${block.x} ${block.y} ${block.z} 10 0.8`)
        addItem(player,itemId,1)
    })

    eventData.cancel = true
});

//Glass Bottle Interaction

world.beforeEvents.playerInteractWithBlock.subscribe(eventData => {
    const block = eventData.block;
    const item = eventData.itemStack;
    const player = eventData.player;

    if (!block.hasTag("CrushingTub"))return;

    if (item == undefined) return;

    if (item.typeId != "minecraft:glass_bottle" && item.typeId != "minecraft:bucket") return;


    const blockState = block.permutation.getState("r:type");
    const liquidState = block.permutation.getState("r:liquid");
    const itemState = block.permutation.getState("r:place_item");

    if (item.typeId == "minecraft:glass_bottle" && liquidState < 2) return;

    if (item.typeId == "minecraft:bucket" && liquidState < 6) return;

    if(blockState == 'none') return;

    if(liquidState < 1) return;

    let itemToFill = undefined

    switch(blockState){
        case 'wildberries':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "rustic:bottle_of_wildberry_juice";
            else if (item.typeId == "minecraft:bucket")itemToFill = "rustic:wildberry_juice_bucket";
            break;
        case 'ironberries':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "rustic:bottle_of_ironberry_juice";
            else if (item.typeId == "minecraft:bucket")itemToFill = "rustic:ironberry_juice_bucket";
            break;
        case 'grapes':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "rustic:bottle_of_grape_juice";
            else if (item.typeId == "minecraft:bucket")itemToFill = "rustic:grape_juice_bucket";
            break;
        case 'olives':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "rustic:bottle_of_olive_oil";
            else if (item.typeId == "minecraft:bucket")itemToFill = "rustic:olive_oil_bucket";
            break;
        case 'apple':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "rustic:bottle_of_apple_juice";
            else if (item.typeId == "minecraft:bucket")itemToFill = "rustic:apple_juice_bucket";
            break;
        case 'honeycomb':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "minecraft:honey_bottle";
            else if (item.typeId == "minecraft:bucket")itemToFill = "rustic:honey_bucket";
            break;
        case 'sugar_cane':
            if (item.typeId == "minecraft:glass_bottle")itemToFill = "minecraft:potion";
            else if (item.typeId == "minecraft:bucket")itemToFill = "minecraft:water_bucket";
            break;
    }
    const height = item.typeId == "minecraft:glass_bottle" ? 2 : 6
    const sound = item.typeId == "minecraft:glass_bottle" ? "bottle.fill" : "cauldron.takepotion"

    system.run( () => {
        if (itemState == 0 && liquidState == 2){
            block.setPermutation(block.permutation.withState("r:type", 'none'));
        }
        block.setPermutation(block.permutation.withState("r:liquid", liquidState - height));
        block.dimension.runCommandAsync(`playsound ${sound} @a ${block.x} ${block.y} ${block.z} 10 0.8`)

        const equipment = player.getComponent('equippable');
        let newitem = undefined;
        if (item.amount == 1){
            newitem = new ItemStack(itemToFill, 1);
            equipment.setEquipment('Mainhand', newitem);
        }
        else{
            if (player.getGameMode() != "creative")item.amount -= 1;
            equipment.setEquipment('Mainhand', item);
            player.getComponent("inventory").container.addItem(new ItemStack(itemToFill, 1))
        }
    })

    eventData.cancel = true
});




export function removeItem(player,item, count){
    const equipment = player.getComponent('equippable');
    system.run( () => {
        if (player.getGameMode() !== "creative") {
            if (item.amount > count) {
                item.amount -= count;
                equipment.setEquipment('Mainhand', item);
            } else if (item.amount === count) {
                equipment.setEquipment('Mainhand', undefined);
            }
        }
    })
}

export function addItem(player,item,count){
    player.getComponent('inventory').container.addItem(new ItemStack(item, count));
}

