import { world, BlockPermutation } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe((initEvent) => {
    initEvent.blockComponentRegistry.registerCustomComponent("rustic:lattice", {
        onPlace: e => {
            const { player, block, previousBlock } = e;

            const directions = {
                north: "south",
                south: "north",
                west: "east",
                east: "west",
                above: "above",
                below: "below",
            };

            Object.entries(directions).forEach(([direction, opposite]) => {
                const adjacentBlock = block[direction]();
                if (isRelevantBlock(adjacentBlock)) {
                    if (adjacentBlock.hasTag("lattice")) {
                        if (direction == "above")direction = "below"
                        else if (direction == "below")direction = "above"
                        adjacentBlock.setPermutation(adjacentBlock.permutation.withState(`r:${direction}`, true));
                    }
                    block.setPermutation(block.permutation.withState(`r:${opposite}`, true));
                }
            });

            
        },
        onPlayerDestroy: e => {
            const { player, block, dimension } = e;
            if ( block.north().hasTag("lattice") ) {
                block.north().setPermutation(block.north().permutation.withState("r:north",false));
            }
            if ( block.south().hasTag("lattice") ) {
                block.south().setPermutation(block.south().permutation.withState("r:south",false));
            }
            if ( block.west().hasTag("lattice") ) {
                block.west().setPermutation(block.west().permutation.withState("r:west",false));
            }
            if ( block.east().hasTag("lattice") ) {
                block.east().setPermutation(block.east().permutation.withState("r:east",false));
            }
            if ( block.above().hasTag("lattice") ) {
                block.above().setPermutation(block.above().permutation.withState("r:below",false));
            }
            if ( block.below().hasTag("lattice") ) {
                block.below().setPermutation(block.below().permutation.withState("r:above",false));
            }
        },
    });
});


world.afterEvents.playerBreakBlock.subscribe(({ player, block }) => {
    if ( block.north().hasTag("lattice") ) {
        block.north().setPermutation(block.north().permutation.withState("r:north",false));
    }
    if ( block.south().hasTag("lattice") ) {
        block.south().setPermutation(block.south().permutation.withState("r:south",false));
    }
    if ( block.west().hasTag("lattice") ) {
        block.west().setPermutation(block.west().permutation.withState("r:west",false));
    }
    if ( block.east().hasTag("lattice") ) {
        block.east().setPermutation(block.east().permutation.withState("r:east",false));
    }
    if ( block.above().hasTag("lattice") ) {
        block.above().setPermutation(block.above().permutation.withState("r:below",false));
    }
    if ( block.below().hasTag("lattice") ) {
        block.below().setPermutation(block.below().permutation.withState("r:above",false));
    }
});

world.afterEvents.playerPlaceBlock.subscribe(({ player, block }) => {
    if(isRelevantBlock(block) && !block.hasTag("lattice")){
        if ( block.north().hasTag("lattice") ) {
            block.north().setPermutation(block.north().permutation.withState("r:north",true));
            
        }
        if ( block.south().hasTag("lattice")) {
            block.south().setPermutation(block.south().permutation.withState("r:south",true));
            
        }
        if ( block.west().hasTag("lattice") ) {
            block.west().setPermutation(block.west().permutation.withState("r:west",true));
            
        }
        if ( block.east().hasTag("lattice")) {
            block.east().setPermutation(block.east().permutation.withState("r:east",true));
            
        }
        if ( block.above().hasTag("lattice")) {
            block.above().setPermutation(block.above().permutation.withState("r:below",true));
            
        }
        if ( block.below().hasTag("lattice")) {
            block.below().setPermutation(block.below().permutation.withState("r:above",true));
            
        }
    }
    
});




export const isRelevantBlock = (adjacentBlock) =>
    adjacentBlock.hasTag("lattice") ||
    (!adjacentBlock.isAir &&
        !adjacentBlock.hasTag("crop") &&
        !adjacentBlock.hasTag("plant") &&
        !adjacentBlock.hasTag("one_way_collidable") &&
        !adjacentBlock.hasTag("rail") &&
        !adjacentBlock.hasTag("snow") &&
        !adjacentBlock.isLiquid &&
        !adjacentBlock.hasTag("text_sign") &&
        !adjacentBlock.hasTag("rustic:non_connectable") &&
        adjacentBlock.typeId != "minecraft:short_grass" &&
        adjacentBlock.typeId != "minecraft:dandelion") &&
        adjacentBlock.typeId != "minecraft:poppy" &&
        adjacentBlock.typeId != "minecraft:blue_orchid" &&
        adjacentBlock.typeId != "minecraft:allium" &&
        adjacentBlock.typeId != "minecraft:azure_bluet" &&
        adjacentBlock.typeId != "minecraft:red_tulip" &&
        adjacentBlock.typeId != "minecraft:orange_tulip" &&
        adjacentBlock.typeId != "minecraft:cornflower";